package ch.sahits.game.graphic.image;

import java.awt.*;
import java.awt.image.*;
import java.util.*;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.io.*;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

import javax.imageio.*;
import javax.swing.*; // for ImageIcon

/**
 * The Imagesfile and images are stored in "Images/" (the IMAGE_DIR constant).<br/>
 * <br/>
 * ImagesFile Formats:<br/>
 * 
 * <pre>
 *     o &lt;fnm&gt;                     // a single image file
 * 
 *     n &lt;fnm*.ext&gt; &lt;number&gt;       // a series of numbered image files, whose
 *                                 // filenames use the numbers 0 - &lt;number&gt;-1
 * 
 *     s &lt;fnm&gt; &lt;number&gt;            // a strip file (fnm) containing a single row
 *                                 // of &lt;number&gt; images
 * 
 *     g &lt;name&gt; &lt;fnm&gt; [ &lt;fnm&gt; ]*   // a group of files with different names;
 *                                 // they are accessible via  
 *                                 // &lt;name&gt; and position _or_ &lt;fnm&gt; prefix
 * </pre>
 * 
 * and blank lines and comment lines.<br/>
 * <br/>
 * The numbered image files (n) can be accessed by the &lt;fnm&gt; prefix and
 * &lt;number&gt;. <br/>
 * <br/>
 * The strip file images can be accessed by the <fnm> prefix and their position
 * inside the file (which is assumed to hold a single row of images).<br/>
 * <br/>
 * The images in group files can be accessed by the 'g' &lt;name&gt; and the
 * &lt;fnm&gt; prefix of the particular file, or its position in the group.<br/>
 * <br/>
 * <br/>
 * The images are stored as BufferedImage objects, so they will be manipulated
 * as 'managed' images by the JVM (when possible).
 * 
 * @author Andrew Davison, April 2005, ad@fivedots.coe.psu.ac.th
 * 
 */
public class ImagesLoader {
	protected final static String IMAGE_DIR = "/";

	/**
	 * The key is the filename prefix, the object (value) is an ArrayList of
	 * BufferedImages
	 */
	protected HashMap<String, List<BufferedImage>> imagesMap;
	/**
	 * The key is the 'g' <name> string, the object is an ArrayList of filename
	 * prefixes for the group. This is used to access a group image by its 'g'
	 * name and filename.
	 */
	private HashMap<String, List<String>> gNamesMap;

	private GraphicsConfiguration gc;

	public ImagesLoader(String fnm)
	// begin by loading the images specified in fnm
	{
		initLoader();
		loadImagesFile(fnm);
	} // end of ImagesLoader()

	public ImagesLoader() {
		initLoader();
	}

	protected void initLoader() {
		imagesMap = new HashMap<String, List<BufferedImage>>();
		gNamesMap = new HashMap<String, List<String>>();

		GraphicsEnvironment ge = GraphicsEnvironment
				.getLocalGraphicsEnvironment();
		gc = ge.getDefaultScreenDevice().getDefaultConfiguration();
	} // end of initLoader()

	/**
	 * Loading the files as specified in the class description
	 * 
	 * @param fnm
	 */
	protected void loadImagesFile(String fnm) {
		String imsFNm = IMAGE_DIR + fnm;
		System.out.println("Reading file: " + imsFNm);
		try {
			InputStream in = this.getClass().getResourceAsStream(imsFNm);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			// BufferedReader br = new BufferedReader( new FileReader(imsFNm));
			parseImageDefinitionsAndLoad(br);
			br.close();
		} catch (IOException e) {
			System.out.println("Error reading file: " + imsFNm);
			System.exit(1);
		}
	} // end of loadImagesFile()
	/**
	 * Parse the definition file and load the corresponding images
	 * @param br
	 * @throws IOException
	 */
	protected void parseImageDefinitionsAndLoad(BufferedReader br)
			throws IOException {
		String line;
		char ch;
		while ((line = br.readLine()) != null) {
			if (line.length() == 0) // blank line
				continue;
			if (line.startsWith("//")) // comment
				continue;
			ch = Character.toLowerCase(line.charAt(0));
			if (ch == 'o') // a single image
				getFileNameImage(line);
			else if (ch == 'n') // a numbered sequence of images
				getNumberedImages(line);
			else if (ch == 's') // an images strip
				getStripImages(line);
			else if (ch == 'g') // a group of images
				getGroupImages(line);
			else
				System.out.println("Do not recognize line: " + line);
		}
	}

	// --------- load a single image -------------------------------

	/**
	 * format: o &lt;fnm&gt;
	 */
	protected final void getFileNameImage(String line)
	{
		StringTokenizer tokens = new StringTokenizer(line);

		if (tokens.countTokens() != 2)
			System.out.println("Wrong no. of arguments for " + line);
		else {
			tokens.nextToken(); // skip command label
			System.out.print("o Line: ");
			loadSingleImage(tokens.nextToken());
		}
	} // end of getFileNameImage()
	/**
	 * Replace a single image entry with another image
	 * @param name under which the image is stored
	 * @param img Image that replaces the old one.
	 * @return false if img is null
	 */
	public boolean replaceSingleImage(String name,BufferedImage img){
		if (img != null) {
			ArrayList<BufferedImage> imsList = new ArrayList<BufferedImage>();
			imsList.add(img);
			imagesMap.put(name, imsList);
			System.out.println("  Replaced " + name );
			return true;
		} else
			return false;
	}
	
	public boolean loadSingleImage(String fnm)
	// can be called directly
	{
		String name = getPrefix(fnm);

		if (imagesMap.containsKey(name)) {
			System.out.println("Error: " + name + "already used");
			return false;
		}

		BufferedImage bi = loadImage(fnm);
		if (bi != null) {
			ArrayList<BufferedImage> imsList = new ArrayList<BufferedImage>();
			imsList.add(bi);
			imagesMap.put(name, imsList);
			System.out.println("  Stored " + name + "/" + fnm);
			return true;
		} else
			return false;
	} // end of loadSingleImage()

	private String getPrefix(String fnm)
	// extract name before '.' of filename
	{
		int posn;
		if ((posn = fnm.lastIndexOf(".")) == -1) {
			System.out.println("No prefix found for filename: " + fnm);
			return fnm;
		} else
			return fnm.substring(0, posn);
	} // end of getPrefix()

	// --------- load numbered images -------------------------------

	protected final void getNumberedImages(String line)
	/*
	 * format: n <fnm*.ext> <number>
	 */
	{
		StringTokenizer tokens = new StringTokenizer(line);

		if (tokens.countTokens() != 3)
			System.out.println("Wrong no. of arguments for " + line);
		else {
			tokens.nextToken(); // skip command label
			System.out.print("n Line: ");

			String fnm = tokens.nextToken();
			int number = -1;
			try {
				number = Integer.parseInt(tokens.nextToken());
			} catch (Exception e) {
				System.out.println("Number is incorrect for " + line);
			}

			loadNumImages(fnm, number);
		}
	} // end of getNumberedImages()

	public int loadNumImages(String fnm, int number)
	/*
	 * Can be called directly. fnm is the filename argument in: n <f*.ext>
	 * <number>
	 */
	{
		String prefix = null;
		String postfix = null;
		int starPosn = fnm.lastIndexOf("*"); // find the '*'
		if (starPosn == -1) {
			System.out.println("No '*' in filename: " + fnm);
			prefix = getPrefix(fnm);
		} else { // treat the fnm as prefix + "*" + postfix
			prefix = fnm.substring(0, starPosn);
			postfix = fnm.substring(starPosn + 1);
		}

		if (imagesMap.containsKey(prefix)) {
			System.out.println("Error: " + prefix + "already used");
			return 0;
		}

		return loadNumImages(prefix, postfix, number);
	} // end of loadNumImages()

	private int loadNumImages(String prefix, String postfix, int number)
	/*
	 * Load a series of image files with the filename format prefix + <i> +
	 * postfix where i ranges from 0 to number-1
	 */
	{
		String imFnm;
		BufferedImage bi;
		ArrayList<BufferedImage> imsList = new ArrayList<BufferedImage>();
		int loadCount = 0;

		if (number <= 0) {
			System.out.println("Error: Number <= 0: " + number);
			imFnm = prefix + postfix;
			if ((bi = loadImage(imFnm)) != null) {
				loadCount++;
				imsList.add(bi);
				System.out.println("  Stored " + prefix + "/" + imFnm);
			}
		} else { // load prefix + <i> + postfix, where i = 0 to <number-1>
			System.out.print("  Adding " + prefix + "/" + prefix + "*"
					+ postfix + "... ");
			for (int i = 0; i < number; i++) {
				imFnm = prefix + i + postfix;
				if ((bi = loadImage(imFnm)) != null) {
					loadCount++;
					imsList.add(bi);
					System.out.print(i + " ");
				}
			}
			System.out.println();
		}

		if (loadCount == 0)
			System.out.println("No images loaded for " + prefix);
		else
			imagesMap.put(prefix, imsList);

		return loadCount;
	} // end of loadNumImages()

	// --------- load image strip -------------------------------

	protected final void getStripImages(String line)
	/*
	 * format: s <fnm> <number>
	 */
	{
		StringTokenizer tokens = new StringTokenizer(line);

		if (tokens.countTokens() != 3)
			System.out.println("Wrong no. of arguments for " + line);
		else {
			tokens.nextToken(); // skip command label
			System.out.print("s Line: ");

			String fnm = tokens.nextToken();
			int number = -1;
			try {
				number = Integer.parseInt(tokens.nextToken());
			} catch (Exception e) {
				System.out.println("Number is incorrect for " + line);
			}

			loadStripImages(fnm, number);
		}
	} // end of getStripImages()

	public int loadStripImages(String fnm, int number)
	/*
	 * Can be called directly, to load a strip file, <fnm>, holding <number>
	 * images.
	 */
	{
		String name = getPrefix(fnm);
		if (imagesMap.containsKey(name)) {
			System.out.println("Error: " + name + "already used");
			return 0;
		}
		// load the images into an array
		BufferedImage[] strip = loadStripImageArray(fnm, number);
		if (strip == null)
			return 0;

		ArrayList<BufferedImage> imsList = new ArrayList<BufferedImage>();
		int loadCount = 0;
		System.out.print("  Adding " + name + "/" + fnm + "... ");
		for (int i = 0; i < strip.length; i++) {
			loadCount++;
			imsList.add(strip[i]);
			System.out.print(i + " ");
		}
		System.out.println();

		if (loadCount == 0)
			System.out.println("No images loaded for " + name);
		else
			imagesMap.put(name, imsList);

		return loadCount;
	} // end of loadStripImages()

	// ------ grouped filename seq. of images ---------

	protected final void getGroupImages(String line)
	/*
	 * format: g <name> <fnm> [ <fnm> ]*
	 */
	{
		StringTokenizer tokens = new StringTokenizer(line);

		if (tokens.countTokens() < 3)
			System.out.println("Wrong no. of arguments for " + line);
		else {
			tokens.nextToken(); // skip command label
			System.out.print("g Line: ");

			String name = tokens.nextToken();

			ArrayList<String> fnms = new ArrayList<String>();
			fnms.add(tokens.nextToken()); // read filenames
			while (tokens.hasMoreTokens())
				fnms.add(tokens.nextToken());

			loadGroupImages(name, fnms);
		}
	} // end of getGroupImages()

	/**
	 * Can be called directly to load a group of images, whose filenames are
	 * stored in the ArrayList <fnms>. They will be stored under the 'g' name
	 * <name>.
	 */
	public int loadGroupImages(String name, ArrayList<String> fnms)
	{
		if (imagesMap.containsKey(name)) {
			System.out.println("Error: " + name + "already used");
			return 0;
		}

		if (fnms.size() == 0) {
			System.out.println("List of filenames is empty");
			return 0;
		}

		BufferedImage bi;
		ArrayList<String> nms = new ArrayList<String>();
		ArrayList<BufferedImage> imsList = new ArrayList<BufferedImage>();
		String nm, fnm;
		int loadCount = 0;

		System.out.println("  Adding to " + name + "...");
		System.out.print("  ");
		for (int i = 0; i < fnms.size(); i++) { // load the files
			fnm = (String) fnms.get(i);
			nm = getPrefix(fnm);
			if ((bi = loadImage(fnm)) != null) {
				loadCount++;
				imsList.add(bi);
				nms.add(nm);
				System.out.print(nm + "/" + fnm + " ");
			}
		}
		System.out.println();

		if (loadCount == 0)
			System.out.println("No images loaded for " + name);
		else {
			imagesMap.put(name, imsList);
			gNamesMap.put(name, nms);
		}

		return loadCount;
	} // end of loadGroupImages()

	public int loadGroupImages(String name, String[] fnms)
	// supply the group filenames in an array
	{
		ArrayList<String> al = new ArrayList<String>(Arrays.asList(fnms));
		return loadGroupImages(name, al);
	}

	// ------------------ access methods -------------------

	public BufferedImage getImage(String name)
	/*
	 * Get the image associated with <name>. If there are several images stored
	 * under that name, return the first one in the list.
	 */
	{
		List<BufferedImage> imsList = imagesMap.get(name);
		if (imsList == null) {
			System.out.println("No image(s) stored under " + name);
			return null;
		}

		// System.out.println("Returning image stored under " + name);
		return (BufferedImage) imsList.get(0);
	} // end of getImage() with name input;

	public BufferedImage getImage(String name, int posn)
	/*
	 * Get the image associated with <name> at position <posn> in its list. If
	 * <posn> is < 0 then return the first image in the list. If posn is bigger
	 * than the list's size, then calculate its value modulo the size.
	 */
	{
		List<BufferedImage> imsList = imagesMap.get(name);
		if (imsList == null) {
			System.out.println("No image(s) stored under " + name);
			return null;
		}

		int size = imsList.size();
		if (posn < 0) {
			// System.out.println("No " + name + " image at position " + posn +
			// "; return position 0");
			return (BufferedImage) imsList.get(0); // return first image
		} else if (posn >= size) {
			// System.out.println("No " + name + " image at position " + posn);
			int newPosn = posn % size; // modulo
			// System.out.println("Return image at position " + newPosn);
			return (BufferedImage) imsList.get(newPosn);
		}

		// System.out.println("Returning " + name + " image at position " +
		// posn);
		return (BufferedImage) imsList.get(posn);
	} // end of getImage() with posn input;

	public BufferedImage getImage(String name, String fnmPrefix)
	/*
	 * Get the image associated with the group <name> and filename prefix
	 * <fnmPrefix>.
	 */
	{
		List<BufferedImage> imsList = imagesMap.get(name);
		if (imsList == null) {
			System.out.println("No image(s) stored under " + name);
			return null;
		}

		int posn = getGroupPosition(name, fnmPrefix);
		if (posn < 0) {
			// System.out.println("Returning image at position 0");
			return (BufferedImage) imsList.get(0); // return first image
		}

		// System.out.println("Returning " + name +
		// " image with pair name " + fnmPrefix);
		return (BufferedImage) imsList.get(posn);
	} // end of getImage() with fnmPrefix input;

	private int getGroupPosition(String name, String fnmPrefix)
	/*
	 * Search the hashmap entry for <name>, looking for <fnmPrefix>. Return its
	 * position in the list, or -1.
	 */
	{
		List<String> groupNames = gNamesMap.get(name);
		if (groupNames == null) {
			System.out.println("No group names for " + name);
			return -1;
		}

		String nm;
		for (int i = 0; i < groupNames.size(); i++) {
			nm = (String) groupNames.get(i);
			if (nm.equals(fnmPrefix))
				return i; // the posn of <fnmPrefix> in the list of names
		}

		System.out.println("No " + fnmPrefix + " group name found for " + name);
		return -1;
	} // end of getGroupPosition()

	public List<BufferedImage> getImages(String name)
	// return all the BufferedImages for the given name
	{
		List<BufferedImage> imsList = imagesMap.get(name);
		if (imsList == null) {
			System.out.println("No image(s) stored under " + name);
			return null;
		}

		System.out.println("Returning all images stored under " + name);
		return imsList;
	} // end of getImages();

	public boolean isLoaded(String name)
	// is <name> a key in the imagesMap hashMap?
	{
		List<BufferedImage> imsList = imagesMap.get(name);
		if (imsList == null)
			return false;
		return true;
	} // end of isLoaded()

	public int numImages(String name)
	// how many images are stored under <name>?
	{
		List<BufferedImage> imsList = imagesMap.get(name);
		if (imsList == null) {
			System.out.println("No image(s) stored under " + name);
			return 0;
		}
		return imsList.size();
	} // end of numImages()

	// ------------------- Image Input ------------------

	/*
	 * There are three versions of loadImage() here! They use: ImageIO // the
	 * preferred approach ImageIcon Image We assume that the BufferedImage copy
	 * required an alpha channel in the latter two approaches.
	 */
	/**
	 * check if an image is available
	 */
	protected final boolean isAvailable(String fnm){
		try {
			BufferedImage im = ImageIO.read(getClass().getResource(IMAGE_DIR + fnm));
			if (im==null){
				return false;
			} else {
				return true;
			}
		} catch (IOException e) {
			return false;
		}
	}
	/**
	 * check if this class is contained in a jar file.
	 * @return true if this class was loaded from a jar file
	 */
	private boolean isInJarFile(){
		URL res =getClass().getResource(getClass().getSimpleName()+".class");
		return "jar".equals(res.getProtocol());
	}
	/**
	 * retrieve all filenames that match the corresponding filter in the image directory.
	 * If the images are contained in a jar file, this method expects, that this class
	 * is contained in the same jar file. If that is not the case, the files will not be found.
	 * @param filenamefilter something of the form firstfilenamepart*counter.ext
	 * @return List of filenames
	 * @throws URISyntaxException 
	 */
	protected final String[] getAvailableFilenames(String filenamefilter){
		if (!isInJarFile()){
			return getAvailableFilenamesFromFileSystem(filenamefilter);
		} else {
			return getAvailableFilenamesFromJar(filenamefilter);
		}
	}

	/**
	 * retrieve all filenames that match the corresponding filter in the image directory
	 * within the jarfile of this class.
	 * @param filenamefilter something of the form firstfilenamepart*counter.ext
	 * @return List of filenames
	 * @throws URISyntaxException 
	 */
	private String[] getAvailableFilenamesFromJar(String filenamefilter) {
		boolean isRoot = IMAGE_DIR.equals("/") || IMAGE_DIR.equals("\\");
		final String head = filenamefilter.substring(0, filenamefilter.indexOf('*'));
		final String tail = filenamefilter.substring(filenamefilter.indexOf('*')+1);
		File f = findJar(getClass());
		Vector<String> fileNames = new Vector<String>();
		try {
			JarFile jFile = new JarFile(f);
			Enumeration<JarEntry> en = jFile.entries();
			while (en.hasMoreElements()) {
				JarEntry entry = (JarEntry) en.nextElement();
				if (!entry.isDirectory()){
					String name = entry.getName();
					if (isRoot){
						if (!name.contains("/") && !name.contains("\\")){
							if (name.startsWith(head) && name.endsWith(tail)){
								fileNames.add(name);
							}
						}
					} else {
						final String imageDir;
						if (IMAGE_DIR.startsWith("/")){
							imageDir= IMAGE_DIR.substring(1);
						} else {
							imageDir = IMAGE_DIR;
						}
						if (name.startsWith(imageDir) && name.indexOf('/', imageDir.length())==-1){
							if (name.startsWith(head) && name.endsWith(tail)){
								fileNames.add(name);
							}
						}
					}
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
			return new String[0];
		}
		String[] result = new String[fileNames.size()];
		fileNames.copyInto(result);
		return result;
	}
	/**
	 * get the jar file for a given class.
	 * This method does not work well for nested or
	 * anonymous classes.
	 *
	 * @param contextClass class object
	 * @return jar files for the class. May be {@code null}
	 *    if the jar file can not be resolved.
	 * @author Nigjo Iqn
	 */
	private File findJar(Class<?> contextClass)
	{
	  URL selfResource = contextClass.getResource(
	      contextClass.getSimpleName() + ".class");
	  return getJarForResource(selfResource);
	}
	/**
	 * get the jar file for a given resource url.
	 *
	 * @param resource url to a resource inside a jar file.
	 * @return jar file or {@code null} if the url does not
	 *    point to a valid resource inside an existing jar file.
	 * @author Nigjo Iqn
	 */
	private File getJarForResource(URL resource)
	{
	  if(!"jar".equals(resource.getProtocol()))
	    return null;
	  String resPath = resource.getPath();
	  try
	  {
	    //short version:
	    //File jarfile = new File(new URI(
	    //    resPath.substring(0, resPath.lastIndexOf("!/"))));
	    //long version:
	    File jarFile = new File(new URI(resPath));
	    while(jarFile != null && !jarFile.exists())
	    {
	      String path = resPath;
	      if(!path.contains("!/"))
	        return null;
	      resPath = path.substring(0, path.lastIndexOf("!/"));
	      jarFile = new File(new URI(resPath));
	    }
	    //end of long version
	    return jarFile;
	  }
	  catch(URISyntaxException ex)
	  {
		  ex.printStackTrace();
	  }
	  return null;
	}


	/**
	 * retrieve all filenames that match the corresponding filter in the image directory
	 * @param filenamefilter something of the form firstfilenamepart*counter.ext
	 * @return List of filenames
	 * @throws URISyntaxException 
	 */
	private String[] getAvailableFilenamesFromFileSystem(String filenamefilter) {
		try {
			URL url = getClass().getResource(IMAGE_DIR);
			final File imageDir = new File(url.toURI());
			final String head = filenamefilter.substring(0, filenamefilter.indexOf('*'));
			final String tail = filenamefilter.substring(filenamefilter.indexOf('*')+1);
			FilenameFilter filter = new FilenameFilter() {
				
				@Override
				public boolean accept(File dir, String name) {
					if (dir.equals(imageDir)){
						if (name.startsWith(head) && name.endsWith(tail)){
							return true;
						}
					}
					return false;
				}
			};
			return imageDir.list(filter);
		} catch (URISyntaxException e) {
			e.printStackTrace();
			return new String[]{};
		}
	}

	/**
	 * Load the image from <fnm>, returning it as a BufferedImage which is
	 * compatible with the graphics device being used. Uses ImageIO.
	 */
	public BufferedImage loadImage(String fnm)
	{
		try {
			URL url = getClass().getResource(IMAGE_DIR + fnm);
			BufferedImage im;
			try {
				im = ImageIO.read(url);
			} catch (IllegalArgumentException e) {
				System.out.println("Image that cannot be loaded: "+fnm+" at "+url);
				throw e;
			}
			// An image returned from ImageIO in J2SE <= 1.4.2 is
			// _not_ a managed image, but is after copying!

			int transparency = im.getColorModel().getTransparency();
			BufferedImage copy = gc.createCompatibleImage(im.getWidth(),
					im.getHeight(), transparency);
			// create a graphics context
			Graphics2D g2d = copy.createGraphics();
			// g2d.setComposite(AlphaComposite.Src);

			// reportTransparency(IMAGE_DIR + fnm, transparency);

			// copy image
			g2d.drawImage(im, 0, 0, null);
			g2d.dispose();
			return copy;
		} catch (IOException e) {
			System.out.println("Load Image error for " + IMAGE_DIR + "/" + fnm
					+ ":\n" + e);
			return null;
		}
	} // end of loadImage() using ImageIO

	private void reportTransparency(String fnm, int transparency) {
		System.out.print(fnm + " transparency: ");
		switch (transparency) {
		case Transparency.OPAQUE:
			System.out.println("opaque");
			break;
		case Transparency.BITMASK:
			System.out.println("bitmask");
			break;
		case Transparency.TRANSLUCENT:
			System.out.println("translucent");
			break;
		default:
			System.out.println("unknown");
			break;
		} // end switch
	} // end of reportTransparency()

	private BufferedImage loadImage2(String fnm)
	/*
	 * Load the image from <fnm>, returning it as a BufferedImage. Uses
	 * ImageIcon.
	 */
	{
		ImageIcon imIcon = new ImageIcon(getClass()
				.getResource(IMAGE_DIR + fnm));
		if (imIcon == null)
			return null;

		int width = imIcon.getIconWidth();
		int height = imIcon.getIconHeight();
		Image im = imIcon.getImage();

		return makeBIM(im, width, height);
	} // end of loadImage() using ImageIcon

	private BufferedImage makeBIM(Image im, int width, int height)
	// make a BufferedImage copy of im, assuming an alpha channel
	{
		BufferedImage copy = new BufferedImage(width, height,
				BufferedImage.TYPE_INT_ARGB);
		// create a graphics context
		Graphics2D g2d = copy.createGraphics();
		// g2d.setComposite(AlphaComposite.Src);

		// copy image
		g2d.drawImage(im, 0, 0, null);
		g2d.dispose();
		return copy;
	} // end of makeBIM()

	public BufferedImage loadImage3(String fnm)
	/*
	 * Load the image from <fnm>, returning it as a BufferedImage. Use Image.
	 */
	{
		Image im = readImage(fnm);
		if (im == null)
			return null;

		int width = im.getWidth(null);
		int height = im.getHeight(null);

		return makeBIM(im, width, height);
	} // end of loadImage() using Image

	private Image readImage(String fnm)
	// load the image, waiting for it to be fully downloaded
	{
		Image image = Toolkit.getDefaultToolkit().getImage(
				getClass().getResource(IMAGE_DIR + fnm));
		MediaTracker imageTracker = new MediaTracker(new JPanel());

		imageTracker.addImage(image, 0);
		try {
			imageTracker.waitForID(0);
		} catch (InterruptedException e) {
			return null;
		}
		if (imageTracker.isErrorID(0))
			return null;
		return image;
	} // end of readImage()

	public BufferedImage[] loadStripImageArray(String fnm, int number)
	/*
	 * Extract the individual images from the strip image file, <fnm>. We assume
	 * the images are stored in a single row, and that there are <number> of
	 * them. The images are returned as an array of BufferedImages
	 */
	{
		if (number <= 0) {
			System.out.println("number <= 0; returning null");
			return null;
		}

		BufferedImage stripIm;
		if ((stripIm = loadImage(fnm)) == null) {
			System.out.println("Returning null");
			return null;
		}

		int imWidth = (int) stripIm.getWidth() / number;
		int height = stripIm.getHeight();
		int transparency = stripIm.getColorModel().getTransparency();

		BufferedImage[] strip = new BufferedImage[number];
		Graphics2D stripGC;

		// each BufferedImage from the strip file is stored in strip[]
		for (int i = 0; i < number; i++) {
			strip[i] = gc.createCompatibleImage(imWidth, height, transparency);

			// create a graphics context
			stripGC = strip[i].createGraphics();
			// stripGC.setComposite(AlphaComposite.Src);

			// copy image
			stripGC.drawImage(stripIm, 0, 0, imWidth, height, i * imWidth, 0,
					(i * imWidth) + imWidth, height, null);
			stripGC.dispose();
		}
		return strip;
	} // end of loadStripImageArray()

} // end of ImagesLoader class
